package meerkat.pollingstation;

import com.google.common.util.concurrent.FutureCallback;
import com.google.protobuf.BoolValue;
import meerkat.protobuf.PollingStation.*;
/**
 * Created by Arbel on 05/05/2016.
 * An interface for the scanner used by the Polling Station Committee
 * The scanner works as a producer, while the polling station is the consumer
 * That is to say: scans are pushed from the scanner rather than requested by the polling station
 */
public interface PollingStationScanner {

    /**
     * An interface for processing scans (Polling Station side)
     */
    public interface Consumer {

        /**
         * Sets up the connection to the scanner and begins receiving scans
         * @throws Exception when the operation fails
         */
        public void start() throws Exception;

        /**
         * Closes the connection to the scanner
         * @throws Exception when the operation fails
         */
        public void stop() throws Exception;

        /**
         * Subscribes to new scans
         *
         * @param scanCallback is the handler for scanned data
         */
        public void subscribe(FutureCallback<ScannedData> scanCallback);

    }

    /**
     * An interface for submitting scanned data (scanner side)
     */
    public interface Producer {

        /**
         * Sends a scan to all subscribers
         * @param scannedData contains the scanned data
         * @return a BoolValue containing TRUE iff the scanned data has been sent to at least one subscriber
         */
        public BoolValue newScan(ScannedData scannedData);

        /**
         * Notifies subscribers about an error that occurred during scan
         * @param errorMsg is the error that occurred
         * @return a BoolValue containing TRUE iff the error has been sent to at least one subscriber
         */
        public BoolValue reportScanError(ErrorMsg errorMsg);

    }
    
}